/*
  Devil-Elec
  Nano Development Board v1.0
  IDE 1.8.19
  Arduino Nano Every
  19.09.2023 
*/

#include <Streaming.h>
#include <assert.h>
#include <NanoEveryPin.h>
#include <Encoder.h>

struct Daten
{
  int8_t direction;
  int32_t relativ;
  int32_t absolut;
  int32_t absOld {8888};
};
Daten enc1;
Daten enc2;

/* Raster meistens '2' Alps EC11 o.ä., für Alps EM20B '4' passend
   Zwei Bsp. zur Veranschaulichung
         Pin Phase A 
         |  Pin Phase B 
         |  |  Raster (1/2/4)
         |  |  |    RelMin (default '-' signed 32bit)
         |  |  |    |   RelMax (default '+' signed 32bit)
         |  |  |    |   |             */
Encoder <3, 4, 2>          Encoder1;   
Encoder <3, 4, 2, -10, 10> Encoder2;  

void setup (void)
{
  Serial.begin(250000);
  Serial.println(F("\nuC Reset ####\n")); 
  Encoder1.init();  
  Encoder1.enablePullup(); 
  Encoder2.init();
  Encoder2.enablePullup();
}

void loop (void)
{
  updateEncoder();
  ausgabe(Serial);
}

void updateEncoder (void)
{   
  Encoder1.encode();        // Daten aktualisieren
  Encoder2.encode();        // Daten aktualisieren
  
  enc1.direction = Encoder1.getDirection();
  enc1.relativ   = Encoder1.getRelCounter();
  enc1.absolut   = Encoder1.getAbsCounter(); 

  enc2.direction = Encoder2.getDirection();
  enc2.relativ   = Encoder2.getRelCounter();
  enc2.absolut   = Encoder2.getAbsCounter();  
}


void ausgabe (Stream &out)
{
  bool state {false};
  
  if (enc1.absolut != enc1.absOld) {
    state = true;
    enc1.absOld = enc1.absolut;
  }

  if (enc2.absolut != enc2.absOld) {
    state = true;
    enc2.absOld = enc2.absolut;
  }

  if (state) seriellerMonitor(out);
}

void seriellerMonitor (Stream &out)
{  
  Ueberschriftszeile(out);

  out << enc1.direction << '\t' << enc1.relativ << '\t' << enc1.absolut << "\t\t" << 
         enc2.direction << '\t' << enc2.relativ << '\t' << enc2.absolut << endl; 
}

void Ueberschriftszeile (Stream &out)
{
  static uint8_t counter {30};

  if (++counter < 30) return; // Zeilen noch nicht erreicht, Funktion abbrechen

  counter = 0;

  out << (F("DIR")) << '\t' << (F("rel")) << '\t' << (F("abs")) << "\t\t" << 
         (F("DIR")) << '\t' << (F("rel")) << '\t' << (F("abs")) << endl;  
}
